local checkMargin = 1

object = {}
object.__index = object

function object.new(x,y,shapeType,shape,onClick,img)
  assertType(x,"number",1);assertType(y,"number",2);assertType(shapeType,"string",3);assertType(shape,"table",4);assertType(onClick,{"nil","function"},5);assertType(img,{"nil","Image","table"},6)
  if #shape%2 == 1 then error("shape (arg #4) should have even amount of elements",2) end
  local obj = setmetatable({},object)
  obj.x = x
  obj.y = y
  obj.shapeType = shapeType
  obj.shape = shape
  obj.onClick = onClick
  if shapeType == "polygon" then
    for i,n in ipairs(shape) do
      if i % 2 == 1 then
        if obj.minX == nil or n < obj.minX then obj.minX = n end
        if obj.maxX == nil or n > obj.maxX then obj.maxX = n end
      else
        if obj.minY == nil or n < obj.minY then obj.minY = n end
        if obj.maxY == nil or n > obj.maxY then obj.maxY = n end
      end
    end
    if not love.math.isConvex(shape) then
      obj.tris = love.math.triangulate(shape)
    end
  elseif shapeType == "rectangle" then
    obj.minX = shape[1]
    obj.minY = shape[2]
    obj.maxX = shape[1]+shape[3]
    obj.maxY = shape[2]+shape[4]
  else
    error("invalid shape type: '" .. shapeType .. "'",2)
  end
  obj.over = false
  obj.frames = (type(img) == "table") and img or {img} -- is the argument img a table? if so, be img. if not, be a table with img in it.
  obj.frameInd = 1
  return obj
end

function object:pointInside(x,y)
  if x >= self.x+self.minX and x <= self.x+self.maxX and y >= self.y+self.minY and y <= self.y+self.maxY then
    if self.shapeType == "rectangle" then
      return true
    elseif self.shapeType == "polygon" then
      x = x - self.x
      y = y - self.y
      local lx1,ly,lx2 = self.minX-checkMargin,y,self.maxX+checkMargin,y
      if pointInShape(lx1,lx2,x,y,self.shape) then
        return true
      end
    end
  end
  return false
end

function object:mousepressed(x,y,b)
  if b == 1 and self.over then
    doex(self.onClick,self)
    return true
  end
end

function object:mousemoved(x,y,dx,dy)
  if self.onRollOver or self.onRollOut or self.onClick then
    local inside = self:pointInside(x,y)
    if self.over and not inside then
      self.over = false
      doex(self.onRollOut)
    elseif not self.over and inside then
      self.over = true
      doex(self.onRollOver)
    end
  end
end

function object:moveShape(dx,dy) -- move the shape only, without changing the x,y properties
  if self.shapeType == "polygon" then
    for i=1,#self.shape do
      if i%2 == 1 then
        self.shape[i] = self.shape[i] + dx
      else
        self.shape[i] = self.shape[i] + dy
      end
    end
    if self.tris then self.tris = love.math.triangulate(self.shape) end
  elseif self.shapeType == "rectangle" then
    self.shape[1],self.shape[2] = self.shape[1]+dx,self.shape[2]+dy
  end
  self.minX,self.maxX = self.minX + dx,self.maxX+dx
  self.minY,self.maxY = self.minY + dy,self.maxY+dy
end

function object:draw()
  if #self.frames > 0 then
    local dx,dy =
    self.x + ((self.shapeType == "rectangle") and self.shape[1] or 0),
    self.y + ((self.shapeType == "rectangle") and self.shape[2] or 0)
    --love.graphics.draw(self.img,dx,dy)
    love.graphics.setColor(1,1,1)
    love.graphics.draw(self.frames[self.frameInd],dx,dy)
  end
  if debugDraw then
    love.graphics.push()
    love.graphics.translate(self.x,self.y)
    love.graphics.setLineWidth(2)
    love.graphics.setColor(1,0,0)
    if self.shapeType == "rectangle" then
      love.graphics.rectangle("line",self.shape[1],self.shape[2],self.shape[3],self.shape[4])
    elseif self.shapeType == "polygon" then
      if self.tris then
        for _,t in ipairs(self.tris) do
          love.graphics.polygon("line",t)
        end
      else
        love.graphics.polygon("line",self.shape)
      end
    end
    love.graphics.pop()
  end
  --[[if self.clicked then love.graphics.setColor(1,1,0,0.5) else love.graphics.setColor(1,0,0,0.5) end]]
end