function assertTypeSingle(x,t,n)
  local tt = type(x)
  if tt == "userdata" and x.type then tt = x:type() end -- if this is a love object, call its type method
  if tt ~= t then error((n and ("arg #" .. n .. " ") or "") .. "expected '" .. t .. "', got '" .. tt .. "'.",4) end
end

function assertTypeMult(x,t,n)
  assertType(t,"table",2);if n then assertType(n,"number",3) end
  local tt
  for _,tn in ipairs(t) do
    tt = type(x)
    if tt == "userdata" and x.type then tt = x:type() end -- if this is a love object, call its type method
    if tn == tt then
      goto succ
    end
  end
  error((n and ("arg #" .. n .. " ") or "") .. "did not expect '" .. tt .. "'.",4)
  ::succ::
end

function assertType(x,t,n)
  if type(t) == "table" then
    assertTypeMult(x,t,n)
  else
    assertTypeSingle(x,t,n)
  end
end

function doex(f,...)
  if f then return f(...) end
end

-----

function lerp(a,b,t)
  return a + (b - a) * t
end

function ccw(x1,y1,x2,y2,x3,y3)
  return (y3-y1) * (x2-x1) > (y2-y1) * (x3-x1)
end

function lineIntersects(x1,y1,x2,y2,x3,y3,x4,y4)
  local x = ((x1 * y2 - y1 * x2) * (x3 - x4) - (x1 - x2) * (x3 * y4 - y3 * x4)) / ((x1 - x2) * (y3 - y4) - (y1 - y2) * (x3 - x4))
  local y = ((x1 * y2 - y1 * x2) * (y3 - y4) - (y1 - y2) * (x3 * y4 - y3 * x4)) / ((x1 - x2) * (y3 - y4) - (y1 - y2) * (x3 - x4))
  if ccw(x1,y1,x3,y3,x4,y4) ~= ccw(x2,y2,x3,y3,x4,y4) and ccw(x1,y1,x2,y2,x3,y3) ~= ccw(x1,y1,x2,y2,x4,y4) then
    return {x=x,y=y}
  end
end

function pointInShape(minX,maxX,px,py,shape)
  assertType(minX,"number");assertType(maxX,"number");assertType(px,"number");assertType(py,"number");assertType(shape,"table");
  if #shape%2 == 1 then error("shape should have even amount of elements",2) end
  local interCount = 0
  for i=1,#shape,2 do
    local x1,y1,x2,y2 = shape[i],shape[i+1],
    ((i+2) > #shape) and shape[1] or shape[i+2],
    ((i+3) > #shape) and shape[2] or shape[i+3]
    local inter = lineIntersects(minX,py,maxX,py,x1,y1,x2,y2)
    if inter then
      if inter.x < px then
        interCount = interCount + 1
      end
    end
  end
  return interCount % 2 == 1
end